@@ -26,6 +26,7 @@ import re
 import shutil
 import StringIO
 import sys
+from termcolor import colored
 import yaml
 from requests.exceptions import ConnectionError
 from ferry.table.prettytable import *
@@ -39,6 +40,7 @@ class CLI(object):
         self.cmds.version = ferry.__version__
         self.cmds.usage = "ferry COMMAND [arg...]"
         self.cmds.add_option("-c", "--conf", "Deployment configuration")
+        self.cmds.add_option("-n", "--dns", "Use custom DNS")
         self.cmds.add_option("-l", "--log", "Log configuration file")
         self.cmds.add_option("-k", "--key", "Specify key directory")
         self.cmds.add_option("-m", "--mode", "Deployment mode")
@@ -74,7 +76,6 @@ class CLI(object):
         payload = { 'payload' : json.dumps(stack_description),
                     'mode' : mode, 
                     'conf' : conf }
-
         try:
             res = requests.post(self.ferry_server + '/create', data=payload)
             return str(res.text)
@@ -82,6 +83,18 @@ class CLI(object):
             logging.error("could not connect to ferry server")
             return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
 
+    def _resubmit_create(self, reply, stack_description, args):
+        values = {}
+        for q in reply['questions']:
+            question = colored(q['query']['text'], 'red')
+            prompt = colored(' >> ', 'green')
+            v = raw_input(question + prompt)
+            values[q['query']['id']] = v
+        stack_description['iparams'] = values
+        reply = self._create_stack(stack_description, args)
+        reply = json.loads(reply)
+        return reply['text']
+
     def _format_snapshots_query(self, json_data):
         bases = []
         date = []
@@ -422,7 +435,12 @@ class CLI(object):
 
                 json_arg['_file_path'] = file_path
             json_arg['_file'] = arg
-            return self._create_stack(json_arg, args)
+            reply = self._create_stack(json_arg, args)
+            reply = json.loads(reply)
+            if reply['status'] == 'query':
+                return self._resubmit_create(reply, json_arg, args)
+            else:
+                return reply['text']
         elif(cmd == 'ps'):
             if len(args) > 0 and args[0] == '-a':
                 opt = args.pop(0)
@@ -437,11 +455,11 @@ class CLI(object):
             return msg
         elif(cmd == 'clean'):
             self._check_ssh_key()
-            self.installer._start_docker_daemon()
+            self.installer.start_web()
+            self.installer._clean_rules()
             self.installer._clean_web()
             self.installer._stop_docker_daemon(force=True)
             self.installer._reset_ssh_key()
-            self.installer._clean_rules()
             return 'cleaned ferry'
         elif(cmd == 'inspect'):
             return self._inspect_stack(args[0])
