@@ -2,22 +2,9 @@
 
 # Make sure we get the env variables in place. 
 source /etc/profile
+source /service/sbin/pophosts
 
-# Copy all the instance addresses and empty the file
-# afterwards so we only copy it over once. 
-input=/service/conf/instances
-host=$(hostname)
-while read line
-do
-	split=( $line )
-	name=${split[1]}
-	if [ "$host" != "$name" ]; then
-	    echo $line >> /etc/hosts
-	fi
-done < "$input"
-echo '' > /service/conf/instances
-
-# Create necessary HDFS directories
+pophosts
 if [ $1 == "hadoop" ]; then
 	su ferry -c '/service/packages/hadoop/bin/hdfs dfs -mkdir /tmp'
 	su ferry -c '/service/packages/hadoop/bin/hdfs dfs -chmod g+w /tmp'
