@@ -53,6 +53,11 @@ def _has_ferry_user():
 def _supported_arch():
     return struct.calcsize("P") * 8 == 64
 
+def _supported_lxc():
+    output = Popen("(lxc-version 2>/dev/null || lxc-start --version) | sed 's/.* //'", stdout=PIPE, shell=True).stdout.read()
+    ver = tuple(map(int, (output.strip().split("."))))
+    return ver > (0, 7, 5)
+
 def _supported_python():
     return sys.version_info[0] == 2
 
@@ -114,7 +119,7 @@ class Installer(object):
         
     def _process_ssh_key(self, options):
         global GLOBAL_KEY_DIR
-        if '-k' in options:
+        if options and '-k' in options:
             GLOBAL_KEY_DIR = 'user://' + self.fetch_image_keys(options['-k'][0])
         else:
             GLOBAL_KEY_DIR = 'tmp://' + DEFAULT_KEY_DIR
@@ -129,6 +134,9 @@ class Installer(object):
         if not _supported_python():
             return 'You appear to be running Python3.\nOnly Python2 is supported at the moment.'
 
+        if not _supported_lxc():
+            return 'You appear to be running an older version of LXC.\nOnly versions > 0.7.5 are supported.'
+
         if not _has_ferry_user():
             return 'You do not appear to have the \'docker\' group configured. Please create the \'docker\' group and try again.'
 
@@ -145,7 +153,7 @@ class Installer(object):
         # Start the Ferry docker daemon. If it does not successfully
         # start, print out a msg. 
         logging.warning("all prerequisites met...")
-        start, msg = self._start_docker_daemon()
+        start, msg = self._start_docker_daemon(options)
         if not start:
             logging.error('ferry docker daemon not started')
             return msg
@@ -153,10 +161,10 @@ class Installer(object):
         # Normally we don't want to build the Dockerfiles,
         # but sometimes we may for testing, etc. 
         build = False
-        if '-b' in options:
+        if options and '-b' in options:
             build = True
 
-        if '-u' in options:
+        if options and '-u' in options:
             if len(options['-u']) > 0 and options['-u'][0] != True:
                 logging.warning("performing select rebuild (%s)" % str(options['-u']))
                 self.build_from_list(options['-u'], 
@@ -203,7 +211,10 @@ class Installer(object):
             return True
 
     def start_web(self, options=None):
-        self._start_docker_daemon()
+        start, msg = self._start_docker_daemon(options)
+        if not start:
+            logging.error(msg) 
+            sys.exit(1)
 
         # Check if the user wants to use a specific key directory. 
         self._process_ssh_key(options)
@@ -231,21 +242,6 @@ class Installer(object):
         # Check if there are any other Mongo instances runnig.
         self._clean_web()
 
-        # self.docker = DockerFabric()
-        # container_info = {'image': DEFAULT_DOCKER_REPO + '/mongodb',
-        #                   'type':'mongodb', 
-        #                   'volumes': { DEFAULT_MONGO_DB : '/service/data',
-        #                                DEFAULT_MONGO_LOG : '/service/logs' },
-        #                   'volume_user':DEFAULT_FERRY_OWNER, 
-        #                   'keys': { '/service/keys' : keydir }, 
-        #                   'ports':[],
-        #                   'exposed':[], 
-        #                   'hostname':'ferry-state',
-        #                   'args':None}
-        # containers = self.docker.alloc(container_info)
-        # if len(containers) == 0:
-        #     exit(1)
-
         # Start the Mongo server.
         keydir, _ = self._read_key_dir()
         cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' run -d -v %s:%s -v %s:%s -v %s:%s %s/mongodb' % (keydir, '/service/keys',
@@ -310,6 +306,8 @@ class Installer(object):
         logging.warning("stopping http servers")
         cmd = 'ps -eaf | grep httpapi | awk \'{print $2}\' | xargs kill -15'
         Popen(cmd, stdout=PIPE, shell=True)
+        cmd = 'ps -eaf | grep ferry.ip.dhcp | awk \'{print $2}\' | xargs kill -15'
+        Popen(cmd, stdout=PIPE, shell=True)
 
     def _clean_web(self):
         docker = DOCKER_CMD + ' -H=' + DOCKER_SOCK
@@ -457,7 +455,7 @@ class Installer(object):
         # image = os.path.dirname(f).split("/")[-1]
         if not image in built_images:
             if base == "base":
-                self._pull_image(base)
+                self._pull_image(base, tag='latest')
 
             built_images[image] = True
             self._compile_image(image, repo, os.path.dirname(f), build)
@@ -491,12 +489,19 @@ class Installer(object):
                 sys.stdout.write(out)
                 sys.stdout.flush()
 
-    def _pull_image(self, image):
-        cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' pull %s' % image
+    def _pull_image(self, image, tag=None):
+        if not tag:
+            tag = ferry.__version__
+        cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' pull %s:%s' % (image, tag)
         logging.warning(cmd)
 
         child = Popen(cmd, stdout=PIPE, shell=True)
         self._continuous_print(child)
+
+        # Now tag the image with the 'latest' tag. 
+        cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' tag' + ' %s:%s %s:%s' % (image, tag, image, 'latest')
+        logging.warning(cmd)
+        Popen(cmd, stdout=PIPE, shell=True)
         
     def _compile_image(self, image, repo, image_dir, build=False):
         # Now build the image. 
@@ -534,7 +539,7 @@ class Installer(object):
         output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
         return output.strip() == "btrfs"
 
-    def _start_docker_daemon(self):
+    def _start_docker_daemon(self, options=None):
         # Check if the docker daemon is already running
         try:
             if not self._docker_running():
@@ -543,9 +548,19 @@ class Installer(object):
                     logging.warning("using btrfs backend")
                     bflag = ' -s btrfs'
 
+                # Explicitly supply the DNS.
+                if options and '-n' in options:
+                    logging.warning("using custom dns")
+                    dflag = ''
+                    for d in options['-n']:
+                        dflag += ' --dns %s' % d
+                else:
+                    logging.warning("using public dns")
+                    dflag = ' --dns 8.8.8.8 --dns 8.8.4.4'
+
                 # We need to fix this so that ICC is set to false. 
                 icc = ' --icc=true'
-                cmd = 'nohup ' + DOCKER_CMD + ' -d' + ' -H=' + DOCKER_SOCK + ' -g=' + DOCKER_DIR + ' -p=' + DOCKER_PID + bflag + icc + ' 1>%s  2>&1 &' % DEFAULT_DOCKER_LOG
+                cmd = 'nohup ' + DOCKER_CMD + ' -d' + ' -H=' + DOCKER_SOCK + ' -g=' + DOCKER_DIR + ' -p=' + DOCKER_PID + dflag + bflag + icc + ' 1>%s  2>&1 &' % DEFAULT_DOCKER_LOG
                 logging.warning(cmd)
                 Popen(cmd, stdout=PIPE, shell=True)
 
@@ -562,7 +577,7 @@ class Installer(object):
     def _stop_docker_daemon(self, force=False):
         if force or self._docker_running():
             logging.warning("stopping docker daemon")
-            cmd = 'pkill -f ferry'
+            cmd = 'pkill -f docker-ferry'
             Popen(cmd, stdout=PIPE, shell=True)
             try:
                 os.remove('/var/run/ferry.sock')
